#!/bin/sh
# Z2 backup script v0.01 alpha 0
# Copyright (C) 2010 Marek Vasut <marek.vasut@gmail.com>
#
# This is under GPL v3
#
# Modified by mozzwald http://www.mozzwald.com

# Make sure only root can run our script
if [ "$(id -u)" != "0" ]; then
   echo "This script must be run as root" 1>&2
   exit 1
fi

echo "************************************************"
echo "           Zipit Z2 Flash Backup"
echo "************************************************"
echo ""
echo "This will backup your Zipit flash partitions"
echo ""

flashdefdir=/mnt/mmcblk0p1/u-boot
restoredir="$flashdefdir/`date +%m%d%Y_%H%m%S`"

# make restore directory
mkdir "$restoredir"

# fix mtdblock partitions, create partition list and backup old flash
flashsz=0
while read line; do
        i=`echo $line | grep mtdblock | cut -d " " -f 4`
        if [ -n "$i" ]; then
		# fix the mtdblock
		echo "Fixing /dev/$i"
		blocknum=`echo "$i" | sed -e "s/^.*\(.\)$/\1/"`
		if [ -e "/dev/$i" ]; then
			rm "/dev/$i"
		fi
		mknod "/dev/$i" b 31 "$blocknum"
		# write the partition list
		partsz=`wc -c "/dev/$i" | sed -r "s/[[:blank:]]*([[:digit:]]+).*/\1/"`
		#echo "/dev/$i $partsz" >> "$partlist"
		# Backup Partitions & Create flash.def
		echo "Backing up /dev/$i"
		dd if="/dev/$i" of="$restoredir/$i.bak"
	        mtdmd5=`md5sum "$restoredir/$i.bak" | cut -d " " -f 1`
	        echo "$i.bak $flashsz $mtdmd5" >> "$restoredir"/flash.def
		flashsz=`expr "$flashsz" + "$partsz"`
		mtdcount=`expr "$blocknum" + 1`
	fi
done </proc/partitions

echo "Backup complete!"
