#!/bin/sh
# ZipitZ2 u-boot flash script
# Copyright (C) 2010 Marek Vasut <marek.vasut@gmail.com>
# This is under GPL v3
# Modified by mozzwald http://www.mozzwald.com
# Modified by rkdavis  http://russelldavis.org

# filenames
image=/tmp/image.img
partlist=/tmp/plist.lst
flashdefdir=/mnt/mmcblk0p1/u-boot
flashdefs=flash.def
restoredir="$flashdefdir/`date +%m%d%Y_%H%M%S`"
changelog=change.log

# Move to flashdefdir
cd "$flashdefdir"

# Versions and Dialog settings
. "$flashdefdir/rootfs.ver"
SCRIPTNAME="Z2uFlasher"
SCRIPTVER="v0.2"
DIALOGRC=/mnt/mmcblk0p1/etc/.dialogrc
DIALOGSIZE="12 38"
BACKTITLE="$SCRIPTNAME $SCRIPTVER  |  $ROOTFSNAME $ROOTFSVER"
#BEGIN="--begin 2 1"

# first warning!
dialog $BEGIN --backtitle "$BACKTITLE" --msgbox "WARNING: THIS COULD BRICK YOUR ZIPIT! \
	No one but YOU is responsible if something goes wrong." $DIALOGSIZE

# View the changelog
dialog $BEGIN --backtitle "$BACKTITLE" --no-cancel --inputbox "Would you like to view the change log? (yes/no)" $DIALOGSIZE 2>/tmp/viewlog
viewlog=`cat /tmp/viewlog`
if [ "$viewlog" = "yes" ] || [ "$viewlog" = "y" ]; then
	dialog $BEGIN --backtitle "$BACKTITLE" --textbox "$changelog" 15 44
fi

dialog $BEGIN --backtitle "$BACKTITLE" --infobox "Preparing flash image.." $DIALOGSIZE
if [ -e "$image" ] ; then
	rm -f "$image";
fi
touch "$image"

if [ -e "$partlist" ] ; then
	rm -f "$partlist";
fi
touch "$partlist"

# create the complete image file with proper spacing
offset=0
count=0
while read file start md5; do
	if ! [ -e "$file" ] ; then
		dialog $BEGIN --backtitle "$BACKTITLE" --infobox  "File \"$file\" not found, aborting!" $DIALOGSIZE
		exit 1;
	fi

	# save uboot and env md5's for later
	if [ $count -eq 0 ]; then
		ubootmd5="$md5"
	fi
	if [ $count -eq 1 ]; then
		envmd5="$md5"
	fi

	# check md5sum
	if [ -z "$md5" ] ; then
		dialog $BEGIN --backtitle "$BACKTITLE" --infobox  "md5sum for \"$file\" not found, aborting!" $DIALOGSIZE
		exit 2;
	elif [ "$md5" != "`md5sum $file | cut -d " " -f 1`" ] ; then
		dialog $BEGIN --backtitle "$BACKTITLE" --infobox "md5sum for \"$file\" does not match, aborting!" $DIALOGSIZE
		exit 3;
	fi

	# calculate empty space between parts
	pad=`expr "$start" - "$offset"`

	if [ "$pad" -lt 0 ] ; then
		dialog $BEGIN --backtitle "$BACKTITLE" --infobox "Image components overlap, aborting!" $DIALOGSIZE
		exit 2;
	elif [ "$pad" -gt 0 ] ; then
		# write empty space to image file
		dd if=/dev/zero bs="$pad" count=1 >> "$image" 2>/dev/null;
	fi

	# write part to image file
	cat "$file" >> "$image"
	# calculate next offset
	offset=`wc -c "$image" | sed -r "s/[[:blank:]]*([[:digit:]]+).*/\1/"`;
	#echo "Offset: $offset"
	md5="\0"
	count=`expr "$count" + 1`
done < "$flashdefs"

dialog $BEGIN --backtitle "$BACKTITLE" --infobox "$image ready for flashing." $DIALOGSIZE

dialog $BEGIN --backtitle "$BACKTITLE" --no-cancel --inputbox "Do you want to make a backup of current flash partitions? (yes/no)" $DIALOGSIZE 2>/tmp/flashbackup

backup=`cat /tmp/flashbackup`
if [ "$backup" = "yes" ] || [ "$backup" = "y" ]; then
	# make restore directory
	mkdir "$restoredir"
else
	dialog $BEGIN --backtitle "$BACKTITLE" --msgbox "NOT backing up current flash partitions" $DIALOGSIZE
fi

# fix mtdblock partitions, create partition list and backup old flash
flashsz=0
#for i in `ls -1 /dev/ | egrep "mtdblock[[:digit:]]+"` ; do
while read line; do
        i=`echo $line | grep mtdblock | cut -d " " -f 4`
        if [ -n "$i" ]; then
		# fix the mtdblock
		dialog $BEGIN --backtitle "$BACKTITLE" --infobox "Fixing /dev/$i" $DIALOGSIZE
		blocknum=`echo "$i" | sed -e "s/^.*\(.\)$/\1/"`
		#blocknum=${i#/dev/mtdblock}
		rm "/dev/$i"
		mknod "/dev/$i" b 31 "$blocknum"
		# write the partition list
		partsz=`wc -c "/dev/$i" | sed -r "s/[[:blank:]]*([[:digit:]]+).*/\1/"`
		echo "/dev/$i $partsz" >> "$partlist"

		# Backup Partitions & Create restore.def
		if [ "$backup" = "yes" ] || [ "$backup" = "y" ]; then
			dialog $BEGIN --backtitle "$BACKTITLE" --infobox "Backing up /dev/$i" $DIALOGSIZE
			dd if="/dev/$i" of="$restoredir/$i.bak" &> /dev/null
	        	mtdmd5=`md5sum "$restoredir/$i.bak" | cut -d " " -f 1`
	        	echo "$i.bak $flashsz $mtdmd5" >> "$restoredir"/flash.def
		fi

		# Check md5 of currently installed u-boot and env
		if [ $blocknum -eq 0 ]; then
			thismd5=`md5sum "/dev/$i" | cut -d " " -f 1`
			if [ "$ubootmd5" != "$thismd5" ]; then
				ubootdiff=1
				echo "u-boot to flash is different" >>/tmp/uflash
			else
				ubootdiff=0
				echo "u-boot to flash is the same" >>/tmp/uflash
			fi
		fi
		if [ $blocknum -eq 1 ]; then
			thismd5=`md5sum "/dev/$i" | cut -d " " -f 1`
			if [ "$envmd5" != "$thismd5" ]; then
				envdiff=1
				echo "env to flash is different" >>/tmp/uflash
			else
				envdiff=0
				echo "env to flash is the same" >>/tmp/uflash
			fi
		fi
		flashsz=`expr "$flashsz" + "$partsz"`
		mtdcount=`expr "$blocknum" + 1`
	fi
done </proc/partitions

if [ "$offset" -gt "$flashsz" ] ; then
	dialog $BEGIN --backtitle "$BACKTITLE" --infobox "Image too big for flash, aborting! Offset: $offset FlashSize: $flashsz" $DIALOGSIZE
	sleep 2
	exit 3;
fi

if [ -z "$partlist" ] ; then
	dialog $BEGIN --backtitle "$BACKTITLE" --infobox  "Partition list empty, aborting!" $DIALOGSIZE
	sleep 2
	exit 4;
fi

# uboot is different, recommend to flash
if [ $ubootdiff -eq 1 ]; then
	dialog $BEGIN --backtitle "$BACKTITLE" --no-cancel --inputbox "u-boot currently in flash does not match. Would you like to flash u-boot? (yes/no)" $DIALOGSIZE 2>/tmp/flashuboot
	checkuflash=`cat /tmp/flashuboot`
	if [ "$checkuflash" = "yes" ] || [ "$checkuflash" = "y" ]; then
		flashuboot=1
		dialog $BEGIN --backtitle "$BACKTITLE" --msgbox "u-boot WILL be flashed" $DIALOGSIZE
	else
		flashuboot=0
		dialog $BEGIN --backtitle "$BACKTITLE" --msgbox "u-boot will NOT be flashed" $DIALOGSIZE
	fi
else
	dialog $BEGIN --backtitle "$BACKTITLE" --msgbox "u-boot version is the same, NOT flashing uboot." $DIALOGSIZE
	flashuboot=0
fi

# uboot environment is different, recommend to flash
if [ $envdiff -eq 1 ]; then
	dialog $BEGIN --backtitle "$BACKTITLE" --no-cancel --inputbox "u-boot environment currently in flash does not match. Would you like to flash u-boot environment? (yes/no)" $DIALOGSIZE 2>/tmp/flashenv
	checkenvflash=`cat /tmp/flashenv`
	if [ "$checkenvflash" = "yes" ] || [ "$checkenvflash" = "y" ]; then
		flashenv=1
		dialog $BEGIN --backtitle "$BACKTITLE" --msgbox "u-boot environment WILL be flashed" $DIALOGSIZE
	else
		flashenv=0
		dialog $BEGIN --backtitle "$BACKTITLE" --msgbox "u-boot environment will NOT be flashed" $DIALOGSIZE
	fi
else
	dialog $BEGIN --backtitle "$BACKTITLE" --msgbox "u-boot environment is the same, NOT flashing environment." $DIALOGSIZE
	flashenv=0
fi

dialog $BEGIN --backtitle "$BACKTITLE" --msgbox "Make sure your Zipit is fully charged and you have the AC Adapter plugged in. " $DIALOGSIZE

dialog $BEGIN --backtitle "$BACKTITLE" --no-cancel --inputbox "Last chance before flashing. Are you sure (yes/no)" $DIALOGSIZE 2>/tmp/nogoingback

flash=`cat /tmp/nogoingback`
if [ "$flash" != "yes" ] && [ "$flash" != "y" ]; then
	dialog $BEGIN --backtitle "$BACKTITLE" --msgbox "You didn't enter yes so aborting" $DIALOGSIZE
	exit 5
fi


dialog $BEGIN --backtitle "$BACKTITLE" --infobox "Starting flashing procedure, do NOT POWER OFF!" $DIALOGSIZE
sleep 3

offset=0
while read part size ; do
	blocknum=`echo "$part" | sed -e "s/^.*\(.\)$/\1/"`
	if [ $blocknum -eq 0 ]; then
		partdesc="u-boot"
	elif [ $blocknum -eq 1 ]; then
		partdesc="u-boot Environment"
	elif [ $blocknum -eq 2 ]; then
		partdesc="Kernel"
	elif [ $blocknum -eq 3 ]; then
		partdesc="Filesystem"
	elif [ $blocknum -eq 4 ]; then
		partdesc="u-boot Splash"
	fi
	if [ $blocknum -eq 0 ] && [ $flashuboot -eq 0 ]; then
		dialog $BEGIN --backtitle "$BACKTITLE" --infobox "Skipping $part ($partdesc)" $DIALOGSIZE
		sleep 2
	elif [ $blocknum -eq 1 ] && [ $flashenv -eq 0 ]; then
		dialog $BEGIN --backtitle "$BACKTITLE" --infobox "Skipping $part ($partdesc)" $DIALOGSIZE
		sleep 2
	else
		dialog $BEGIN --backtitle "$BACKTITLE" --infobox "Writing to $part ($partdesc)" $DIALOGSIZE
		#sleep 2
		dd if="$image" of="$part" bs=1 skip="$offset" count="$size"
	fi
	offset=`expr "$offset" + "$size"`;
done < "$partlist"

rm -f "$partlist" "$image"

dialog $BEGIN --backtitle "$BACKTITLE" --no-cancel --inputbox "Flashing complete! The Zipit will boot into the new system after a power cycle.\n\nWould you like to power down now? (yes/no)" $DIALOGSIZE 2>/tmp/pwrdown
pwrdown=`cat /tmp/pwrdown`
if [ "$pwrdown" = "yes" ] || [ "$pwrdown" = "y" ]; then
	dialog $BEGIN --backtitle "$BACKTITLE" --infobox "Shutting down..." $DIALOGSIZE
	sleep 1
	poweroff
else
	dialog $BEGIN --backtitle "$BACKTITLE" --infobox "Exiting to shell..." $DIALOGSIZE
	exit
fi

